classdef EvaluateSimulations <handle
    % This class sets up storage for simulation results, provides a
    % listener and iteration count to store results and functions to
    % evaluate and report summary statistics after simulations are carried
    % out.
    properties
        MSE   = struct('OLS',[],'IV',[],'IVA',[],'GMM',[]);
        MAE   = struct('OLS',[],'IV',[],'IVA',[],'GMM',[]);
        Bias  = struct('OLS',[],'IV',[],'IVA',[],'GMM',[]);
        Med   = struct('OLS',[],'IV',[],'IVA',[],'GMM',[]);
        FSR2  = struct('OLS',[],'IV',[],'IVA',[],'GMM',[]);
		mEig  = struct('OLS',[],'IV',[],'IVA',[],'GMM',[]);
        Iteration = 0;
        RMSE   = struct('OLS',[],'IV',[],'IVA',[],'GMM',[]);
        AMAE   = struct('OLS',[],'IV',[],'IVA',[],'GMM',[]);        
        ABias  = struct('OLS',[],'IV',[],'IVA',[],'GMM',[]);        
        AMed   = struct('OLS',[],'IV',[],'IVA',[],'GMM',[]);                
        AFSR2  = struct('OLS',[],'IV',[],'IVA',[],'GMM',[]);
		AmEig  = struct('OLS',[],'IV',[],'IVA',[],'GMM',[]);
        Outputfilename;

    end
    methods 
        function Results = EvaluateSimulations(NumSim,outfn,firstIni) % function to construct class
            if firstIni
                     Results.Outputfilename = outfn;
                     dlmwrite(Results.Outputfilename,[],'delimiter','');
                     Results.Iteration = 0;
            else
                     Results.Outputfilename = outfn;
                     Results.Iteration = 0;
            end

            Results.MSE.OLS      = zeros(NumSim,1);
            Results.MSE.IV       = zeros(NumSim,1);
            Results.MSE.IVA      = zeros(NumSim,1);
            Results.MSE.GMM      = zeros(NumSim,1);
            
            Results.MAE.OLS      = zeros(NumSim,1);
            Results.MAE.IV       = zeros(NumSim,1);
            Results.MAE.IVA      = zeros(NumSim,1);
            Results.MAE.GMM      = zeros(NumSim,1);
            
            Results.Bias.OLS      = zeros(NumSim,1);
            Results.Bias.IV       = zeros(NumSim,1);
            Results.Bias.IVA      = zeros(NumSim,1);
            Results.Bias.GMM      = zeros(NumSim,1);
            
            Results.Med.OLS      = zeros(NumSim,1);
            Results.Med.IV       = zeros(NumSim,1);
            Results.Med.IVA      = zeros(NumSim,1);
            Results.Med.GMM      = zeros(NumSim,1);
            
            Results.FSR2.OLS      = zeros(NumSim,1);
            Results.FSR2.IV       = zeros(NumSim,1);
            Results.FSR2.IVA      = zeros(NumSim,1);
            Results.FSR2.GMM      = zeros(NumSim,1);
			
			Results.mEig.GMM      = zeros(NumSim,1);
			
        end
        function ySim = CompMSE(Results,ySim,it)
            type           = ySim.Param.type;
            fe             = ySim.lambda - ySim.Param.lambda;
            if it
               Results.Iteration = Results.Iteration + 1; %increment iteration counter 
            end
            switch type
                case 'OLS'
                     Results.MSE.OLS(Results.Iteration)   = fe^2;
                     Results.MAE.OLS(Results.Iteration)   = abs(fe);                     
                     Results.Bias.OLS(Results.Iteration)  = fe;
                     Results.Med.OLS(Results.Iteration)   = fe;                     
                case 'IV'
                     Results.MSE.IV(Results.Iteration)   = fe^2;
                     Results.MAE.IV(Results.Iteration)   = abs(fe);                     
                     Results.Bias.IV(Results.Iteration)  = fe;
                     Results.Med.IV(Results.Iteration)   = fe;                     
                     Results.FSR2.IV(Results.Iteration)  = ySim.R2Dat;
                case 'IVA'
                     Results.MSE.IVA(Results.Iteration)   = fe^2;
                     Results.MAE.IVA(Results.Iteration)   = abs(fe);                     
                     Results.Bias.IVA(Results.Iteration)  = fe;
                     Results.Med.IVA(Results.Iteration)   = fe;                     
                     Results.FSR2.IVA(Results.Iteration)  = ySim.R2Dat;                     
                case 'GMM'
                     Results.MSE.GMM(Results.Iteration)   = fe^2;
                     Results.MAE.GMM(Results.Iteration)   = abs(fe);                     
                     Results.Bias.GMM(Results.Iteration)  = fe;
                     Results.Med.GMM(Results.Iteration)   = fe;                     
                     Results.FSR2.GMM(Results.Iteration)  = ySim.R2Dat;
					 Results.mEig.GMM(Results.Iteration)  = ySim.mEig;
                     
            end
        end
        function Results = AveMSE(Results,y)
            Results.RMSE.OLS      = sqrt(mean(Results.MSE.OLS));
            Results.RMSE.IV       = sqrt(mean(Results.MSE.IV));
            Results.RMSE.IVA      = sqrt(mean(Results.MSE.IVA));
            Results.RMSE.GMM      = sqrt(mean(Results.MSE.GMM));
            
            Results.AMAE.OLS      = mean(Results.MAE.OLS);
            Results.AMAE.IV       = mean(Results.MAE.IV);
            Results.AMAE.IVA      = mean(Results.MAE.IVA);
            Results.AMAE.GMM      = mean(Results.MAE.GMM);

            Results.ABias.OLS      = mean(Results.Bias.OLS);
            Results.ABias.IV       = mean(Results.Bias.IV);
            Results.ABias.IVA      = mean(Results.Bias.IVA);
            Results.ABias.GMM      = mean(Results.Bias.GMM);
            
            Results.AMed.OLS       = median(Results.Bias.OLS);
            Results.AMed.IV        = median(Results.Bias.IV);
            Results.AMed.IVA       = median(Results.Bias.IVA);
            Results.AMed.GMM       = median(Results.Bias.GMM);
         
            Results.AFSR2.IV       = mean(Results.FSR2.IV);
            Results.AFSR2.IVA      = mean(Results.FSR2.IVA);
            Results.AFSR2.GMM      = mean(Results.FSR2.GMM);
			
			Results.AmEig.GMM      = mean(Results.mEig.GMM);

            
            s = num2str(y.Param.n);
            s = [s '&' char(9) num2str(y.Param.lambda) '&'];
            s = [s char(9) num2str(y.Param.amu) '&'];           
            s = [s char(9) num2str(y.Param.LocalID) '&'];      
            
            s = [s char(9) char(9)  '&' char(9)  '&'];            
%            s = [s char(9) num2str(Results.ABias.OLS)]; 
            s = [s char(9) num2str(Results.AMed.OLS,'%-6.3f')  '&'];             
            s = [s char(9) num2str(Results.AMAE.OLS,'%-6.3f') '&'];                  
            s = [s char(9) num2str(Results.RMSE.OLS,'%-6.3f') '&'];      
            
            s = [s char(9) char(9) '&' char(9) '&'];            
%           s = [s char(9) num2str(Results.ABias.IV)];                  
            s = [s char(9) num2str(Results.AMed.IV,'%-6.3f') '&'];                              
            s = [s char(9) num2str(Results.AMAE.IV,'%-6.3f') '&'];                  
            s = [s char(9) num2str(Results.RMSE.IV,'%-6.3f') '&']; 
            s = [s char(9) num2str(Results.AFSR2.IV,'%-6.3f') '&']; 

%            s = [s char(9) char(9) char(9)];            
%%           s = [s char(9) num2str(Results.ABias.IVA)];                  
%            s = [s char(9) num2str(Results.AMed.IVA)];                              
%            s = [s char(9) num2str(Results.AMAE.IVA)];                  
%            s = [s char(9) num2str(Results.RMSE.IVA)]; 
%            s = [s char(9) num2str(Results.AFSR2.IVA)]; 

            
            s = [s char(9) char(9) '&' char(9) '&'];
%            s = [s char(9) num2str(Results.ABias.GMM)];                 
            s = [s char(9) num2str(Results.AMed.GMM,'%-6.3f') '&'];                             
            s = [s char(9) num2str(Results.AMAE.GMM,'%-6.3f') '&'];                  
            s = [s char(9) num2str(Results.RMSE.GMM,'%-6.3f') '&'];      
            s = [s char(9) num2str(Results.AFSR2.GMM,'%-6.3f') '&'];                  
			s = [s char(9) num2str(Results.AmEig.GMM,'%-6.3f') '\\'];                  
             
            dlmwrite(Results.Outputfilename,s,'-append','delimiter','');
        end
    end
 end
