classdef SimulateGI < SimulatedSample
    methods
        function yA = SimulateA(Param)

            yA       = yA@SimulatedSample(Param);            
            n        = yA.Param.n;
            axi      = yA.Param.axi;
            azeta    = yA.Param.azeta;
            a0       = yA.Param.a0;
            ku       = yA.Param.ku;
            betaz    = yA.Param.betaz;
            betazbar = yA.Param.betazbar;
            lambda   = yA.Param.lambda;
            
            zl    = randn(n,1);
            shz   = ceil(rand(n,1)*2*n);
            zl    = zl + shz; %shift covariates to spread them out 
            
            z     = randn(n,yA.Param.d);
            
            zeta  = [zl,z];
            
            eps   = InverseLogisitc(rand(n,n));
            v     = randn(n,1);
            xi    = randn(n,1);
            u     = v + xi;
            
            U     = zeros(n,n);  %utility variables
            M     = zeros(n,n);  %network adjacency matrix
            for i=1:n
                for j=1:n
                    UtilIndex   = azeta*EuclidNorm((zeta(j,:) - zeta(i,:))') + axi*EuclidNorm(xi(i)-xi(j));
                    if UtilIndex < ku
                        U(i,j)  = -1;
                        Uij     = -1;
                    else
                        U(i,j)      = a0 + UtilIndex + eps(i,j); 
                        Uij         = a0 + UtilIndex + eps(j,i); 
                    end
                    M(i,j)      = (U(i,j)>0)*(Uij>0);
                end
                M(i,i)  = 0;
                Ms      = M(i,:)*ones(n,1);   
                if Ms==0
                    M(i,:)  = zeros(1,n);
                else
                    M(i,:)  = M(i,:)./(ones(1,n)*Ms);
                end
            end
            
            y = inv(I-lambda*M)*(z*betaz + M*z*betazbar + u); %reduced form for outcome variable
            
            h = [z,M*z,M*M*z];
             

            yA.yDat    = y;
            yA.zDat    = z;
            yA.zetaDat = zeta;
            yA.MDat    = M;
            yA.hDat    = h;
        end
    end
end
