classdef SimulateGI < SimulatedSample
    methods
        function yA = SimulateGI(Param)

            yA       = yA@SimulatedSample(Param);            
            n        = yA.Param.n;
            amu      = yA.Param.amu;
            kappa    = yA.Param.kappa;
            azeta    = yA.Param.azeta;
            a0       = yA.Param.a0;
            betaz    = yA.Param.betaz;
            betazbar = yA.Param.betazbar;
            lambda   = yA.Param.lambda;
            
            zeta    = rand(n,1)*2;
            %shz     = ceil(rand(n,1)*sqrt(n));
			shz     = (1:n)';
            zeta    = zeta + shz; %shift covariates to spread them out 
            
            z1_1      = randn(n,yA.Param.d);
            z1_2      = randn(n,yA.Param.d);
            
            %zt       = [zl,zeta];
            
            ups       = InverseLogistic(rand(n,n));
            v1        = randn(n,1);
            v2        = randn(n,1);            
            mui       = randn(n,1);
            u1        = v1 + mui;
            u2        = v2 + mui;
            
%           U         = zeros(n,n);  %utility variables
            M         = zeros(n,n);  %network adjacency matrix
            for i=1:n
                zetai  = zeta(i,:);
                mui_i  = mui(i);
                upsi   = ups(i,:);
                Mi     = zeros(1,n);
                sij_cutOff   = sijfunction(zetai,zeta,kappa); % finds cutoff rule to limit number of links at kappa 
                for j=1:n
                    UtilIndex   = azeta*EuclidNorm((zeta(j,:) -zetai)') + amu*EuclidNorm(mui_i-mui(j));
%                   if UtilIndex < -ku
%                        Uji     = -1;
%                        Uij     = -1;
%                   else
                    Uji          = a0 + UtilIndex + upsi(j); 
                    Uij          = a0 + UtilIndex + ups(j,i); 
%                   end
                    sij          = (EuclidNorm((zeta(j,:) -zetai)') < kappa/2); 
                    Mi(j)        = (Uji>0)*(Uij>0)*sij;
                end
                M(i,:)  = Mi;
                M(i,i)  = 0;
                Ms      = M(i,:)*ones(n,1);   
                if Ms==0
                    M(i,:)  = zeros(1,n);
                else
                    M(i,:)  = M(i,:)./(ones(1,n)*Ms);
                end
            end
            
            det(eye(n)-lambda*M)
            
            y1     = inv(eye(n)-lambda*M)*(z1_1*betaz + M*z1_1*betazbar + u1); %reduced form for outcome variable
            y2     = inv(eye(n)-lambda*M)*(z1_2*betaz + M*z1_2*betazbar + u2); %reduced form for outcome variable
            
            yplus  = (y2 - y1)/sqrt(2);
            zplus  = (z1_2 - z1_1)/sqrt(2);
            Mzplus = M*(z1_2 - z1_1)/sqrt(2);
            Zplus  = [zplus,Mzplus];
            Wplus  = M*yplus;
            
            h = [zplus,M*zplus,M*M*zplus];
            

            yA.yDat    = yplus;
            yA.zDat    = zplus;
            yA.ZDat    = Zplus;
            yA.WDat    = Wplus;
            yA.zetaDat = zeta;
            yA.MDat    = M;
            yA.hDat    = h;
        end
    end
end
