classdef SimulatedSample < handle
    properties
        Param = struct('n',[],'amu',[],'kappa',[],'azeta',[],'a0',[],'betaz',[],'betazbar',[],'LocalID',[],'lambda',[],'d',[],'type',[]);
        yDat
        zDat  % basic exogenous covariates
        ZDat  % basic exogenous and group averaged covariates z,Mz
        WDat  % My,z,Mz
        MDat  % adjacency matrix
        hDat
        zetaDat
        delta
        lambda
        beta
        mEig
        R2Dat
        
    end
    methods 
        function ySim = SimulatedSample(Param)
            ySim.Param      = Param;
        end
        function ySim = OLS(ySim)
               z = ySim.zDat;
               y = ySim.yDat;
               M = ySim.MDat;
               W = [M*y,z,M*z];
               ySim.delta = inv(W'*W)*W'*y;  
               ySim.lambda = ySim.delta(1,1);
               ySim.beta   = ySim.delta(2:end,1);               
               ySim.Param.type='OLS';
        end
        function ySim = IV(ySim)             
               h           = ySim.hDat;
               y           = ySim.yDat;
               z           = ySim.zDat;
               M           = ySim.MDat;
               W           = [M*y,z,M*z];
%               W           = [M*y,z];
               Ph          = h*inv(h'*h)*h';
               n           = size(h,1);
               R2          = 1 - (y'*M'*(eye(n)-Ph)*M*y)/(y'*(M'*M)*y);
               ySim.delta  = inv(W'*Ph*W)*W'*Ph*y;
               ySim.lambda = ySim.delta(1,1);
               ySim.beta   = ySim.delta(2:end,1);
               ySim.R2Dat  = R2;
               ySim.Param.type='IV';
        end
        function ySim = IVA(ySim)                  
               y           = ySim.yDat;
               z           = ySim.zDat;
               zeta        = ySim.zetaDat;               
               M           = ySim.MDat;
               W           = [M*y,z,M*z];
%               W           = [M*y,z];
               [A1,A2,~]   = ConstructA_M(M);
               h           = [z,A1*z,A2*z];
               Ph          = h*inv(h'*h)*h';
               n           = size(h,1);
               R2          = 1 - (y'*M'*(eye(n)-Ph)*M*y)/(y'*(M'*M)*y);
               ySim.delta  = inv(W'*Ph*W)*W'*Ph*y;
               ySim.lambda = ySim.delta(1,1);
               ySim.beta   = ySim.delta(2:end,1);
               ySim.R2Dat  = R2;
               ySim.Param.type='IVA';
        end

        function ySim = GMM(ySim)
               h               = ySim.hDat;               
               y               = ySim.yDat;
               z               = ySim.zDat;
               M               = ySim.MDat;
               w               = [z,M*z];
%               w               = [z,M*z];
               n               = length(y);                             
               [A1,A2,Va]      = ConstructA_M(M);
               %h               = [z,A1*z,A2*z];
               Ph              = (h/(h'*h))*h';
               Qh              = eye(n)-(w/(w'*Ph*w))*w'*Ph;      
               R2              = 1 - (y'*M'*(eye(n)-Ph)*M*y)/(y'*(M'*M)*y);
               lambda0         = 0;
               [lambda1,~]     = fminunc(@(lambda)mq(lambda,y,M,Qh,A1,A2,Va),lambda0);
%               lambda1         = mqExp(y,M,Qh,A1,A2,Va);
               beta1           = (w'*Ph*w)\(w'*Ph*(eye(n)-M*lambda1)*y);
			   S1              = [y'*M*Qh*A1*Qh*y,y'*M*Qh*A1*Qh*M*y];
			   S2              = [y'*M*Qh*A2*Qh*y,y'*M*Qh*A2*Qh*M*y];
               S               = [S1;S2];
			   minEig          = min(eig(S'*S));
               ySim.delta      = [lambda1;beta1];
			   [delta1,~]      = fminunc(@(delta)mlinq(delta,y,M,z,Ph,Qh,A1,A2,Va),ySim.delta);
			   ySim.delta      = delta1;
               ySim.lambda     = ySim.delta(1,1);
               ySim.beta       = ySim.delta(2:end,1);
               ySim.R2Dat      = R2;
			   ySim.mEig       = minEig;
               ySim.Param.type = 'GMM';
        end
     end
end
     
                
         
    
           